from typing import Dict, Optional

from abeja.common.api_client import BaseAPIClient


class APIClient(BaseAPIClient):
    """A low-level client for Deployment API

    .. code-block:: python

       from abeja.deployments import APIClient

       api_client = APIClient()
    """

    def create_deployment(
            self, organization_id: str, model_id: str, name: str,
            default_environment: Optional[Dict[str, str]]=None) -> dict:
        """create a deployment

        API reference: POST /organizations/<organization_id>/models/<models_id>/deployments

        Request Syntax:
            .. code-block:: python

                model_id = "1111111111111"
                organization_id = "1111111111111"
                deployment_name = "deployment_name"
                default_environment = {
                    'SAMPLE_ENV': 'SAMPLE_VALUE'
                }

                response = api_client.create_deployment(organization_id, model_id, deployment_name, default_environment)

        Params:
            - **organization_id** (str): organization identifier
            - **model_id** (str): model identifier
            - **name** (str): deploymnet name
            - **default_environment** (dict): default environment variables on the running environment

        Return type:
            dict
        Returns:
            Response Syntax:
                .. code-block:: json

                    {
                        "deployment_id": "1111111111111",
                        "name": "deployment_name",
                        "model_id": "1111111111111",
                        "creator": {
                            "display_name": null,
                            "email": "platform-support@abeja.asia",
                            "id": "1111111111111",
                            "is_registered": true,
                            "role": "admin",
                            "created_at": "2017-05-29T07:48:55Z",
                            "updated_at": "2017-11-29T10:21:24Z"
                        },
                        "default_environment": {},
                        "runs": [],
                        "daemons": [],
                        "services": [],
                        "triggers": [],
                        "created_at": "2018-06-05T08:52:02.428441Z",
                        "modified_at": "2018-06-05T08:52:02.428587Z"
                    }

        Raises:
            - BadRequest: the resource already exists or parameters is insufficient or invalid.
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        payload = {
            'name': name
        }
        if default_environment:
            payload.update({
                'default_environment': default_environment
            })
        path = '/organizations/{}/models/{}/deployments'.format(organization_id, model_id)
        return self._connection.api_request(method='POST', path=path, json=payload)

    def get_deployment(self, organization_id: str, deployment_id: str) -> dict:
        """get a deployment

        API reference: GET /organizations/<organization_id>/deployments/<deployment_id>

        Request Syntax:
            .. code-block:: python

                response = api_client.get_deployment(organization_id='1111111111111', model_id='1111111111111')

        Params:
            - **organization_id** (str): organization_id
            - **deployment_id** (str): deployment_id of the requested model

        Return type:
            dict

        Returns:
            Response Syntax:
                .. code-block:: json

                    {
                        "deployment_id": "1111111111111",
                        "name": "deployment_name",
                        "model_id": "1111111111111",
                        "creator": {
                            "display_name": null,
                            "email": "platform-support@abeja.asia",
                            "id": "1111111111111",
                            "is_registered": true,
                            "role": "admin",
                            "created_at": "2017-05-29T07:48:55Z",
                            "updated_at": "2017-11-29T10:21:24Z"
                        },
                        "default_environment": {},
                        "runs": [],
                        "daemons": [],
                        "services": [],
                        "triggers": [],
                        "created_at": "2018-06-05T08:52:02.428441Z",
                        "modified_at": "2018-06-05T08:52:02.428587Z",
                    }

        Raises:
          - NotFound: model not found
          - Unauthorized: Authentication failed
          - InternalServerError
        """

        path = '/organizations/{}/deployments/{}'.format(organization_id, deployment_id)
        return self._connection.api_request(method='GET', path=path)

    def get_deployments(self, organization_id: str) -> dict:
        """Get deployments entries

        API reference: GET /organizations/<organization_id>/deployments/

        Request syntax:
            .. code-block:: python

                response = api_client.get_deployments(organization_id='1102940376065')

        Params:
            - **organization_id** (str): organization_id

        Return type:
            dict

        Returns:
            Return syntax:
                .. code-block:: json

                    {
                        "entries": [
                            {
                                "deployment_id": "1111111111111",
                                "name": "deployment_name",
                                "model_id": "1111111111111",
                                "creator": {
                                    "display_name": null,
                                    "email": "paltform-support@abeja.asia",
                                    "id": "1111111111111",
                                    "is_registered": true,
                                    "role": "admin",
                                    "created_at": "2017-05-29T07:48:55Z",
                                    "updated_at": "2017-11-29T10:21:24Z"
                                },
                                "default_environment": {},
                                "runs": [],
                                "daemons": [],
                                "services": [],
                                "triggers": [],
                                "created_at": "2018-06-05T08:52:02.428441Z",
                                "modified_at": "2018-06-05T08:52:02.428587Z"
                            }
                        ]
                    }

        Raises:
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        path = '/organizations/{}/deployments'.format(organization_id)
        return self._connection.api_request(method='GET', path=path)

    def delete_deployment(self, organization_id: str, deployment_id: str) -> dict:
        """delete a deployment

        API reference: DELETE /organizations/<organization_id>/deployments/<deployment_id>

        Request syntax:
            .. code-block:: python

                response = api_client.delete_deployment(organization_id='1111111111111', deployment_id='1111111111111')

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **deployment_id** (str): deployment identifier

        Return type:
            dict

        Responses:
            Response syntax:
                .. code-block:: json

                    {
                        "message": "1111111111111 deleted"
                    }

        Raises:
          - NotFound: model not found
          - Unauthorized: Authentication failed
          - InternalServerError
        """
        path = '/organizations/{}/deployments/{}'.format(organization_id, deployment_id)
        return self._connection.api_request(method='DELETE', path=path)

    def patch_deployment(self, organization_id: str, deployment_id: str, name: str, default_environment: Optional[Dict[str, str]]=None):
        """update a deployment

        API reference: PATCH /organizations/<organization_id>/deployments/<deployment_id>

        Request syntax:
            .. code-block:: python

                organization_id = "1111111111111"
                deployment_name = "deployment_name"
                default_environment = {
                    'SAMPLE_ENV': 'SAMPLE_VALUE'
                }

                response = api_client.create_deployment(organization_id, deployment_name, default_environment)

        Params:
            - **organization_id** (str): organization identifier
            - **model_id** (str): model identifier
            - **name** (str): deploymnet name
            - **default_environment** (dict): default environment variables on the running environment

        Return type:
            dict
        Returns:
            Response Syntax:
                .. code-block:: json

                    {
                        "deployment_id": "1111111111111",
                        "name": "deployment_name",
                        "model_id": "1111111111111",
                        "default_environment": {},
                        "runs": [],
                        "daemons": [],
                        "services": [],
                        "triggers": [],
                        "created_at": "2018-06-05T08:52:02.428441Z",
                        "modified_at": "2018-06-05T08:52:02.428587Z"
                    }

        Raises:
            - BadRequest: the resource already exists or parameters is insufficient or invalid.
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        payload = {
            'name': name
        }
        if default_environment is not None:
            payload.update({
                'default_environment': default_environment
            })
        path = '/organizations/{}/deployments/{}'.format(organization_id, deployment_id)
        return self._connection.api_request(method='PATCH', path=path, json=payload)
